# vim:ts=4 sw=4
# ----------------------------------------------------------------------------------------------------
#  Name		: ETL::Pequel3::Type::Properties.pm
#  Created	: 5 May 2006
#  Author	: Mario Gaffiero (gaffie)
#
# Copyright 1999-2007 Mario Gaffiero.
# 
# This file is part of Pequel(TM).
# 
# Pequel is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
# 
# Pequel is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Pequel; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# ----------------------------------------------------------------------------------------------------
# Modification History
# When          Version     Who     What
# ----------------------------------------------------------------------------------------------------
package ETL::Pequel3::Type::Properties;
require 5.005_62;
use strict;
use warnings;
use stl;
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::Abstract;
	use base qw(Class::STL::Element);
	use Class::STL::ClassMembers 
		qw( 
			name 
			default 
			value 
			validate 
			validate_catalogue
			description 
		),
		# validate: use xml-schema types (xs:token, xs:string, etc) or include pattern for xs:pattern
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 1), # -> configuration
		Class::STL::ClassMembers::DataMember->new(name => 'cmd_line', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'required', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'token'),
		Class::STL::ClassMembers::DataMember->new(name => 'CDATA', default => 0);
	use Class::STL::ClassMembers::Constructor;
	sub new_extra
	{
		my $self = shift;
		$self->value($self->default())
			if (defined($self->default()) && !defined($self->value()));
	}
	sub xml_schema
	{
		my $self = shift;
		my $xsd = shift;
		my $xml_node = shift || $xsd->schema_root();
		return if ($xsd->exists('element', $self->name(), $xml_node));
		$xsd->comment($xml_node, "Property Type: @{[ $self->name() ]} [@{[ ref($self) ]}]");
		my $element = $xsd->xs_element($xml_node, 'name' => $self->name());
		if ($xsd->configuration()->xmlschema_annotation() && defined($self->description()) 
			&& $self->description() ne '') 
		{
			my $doc = $xsd->xs_documentation($xsd->xs_annotation($element), 'xml:lang' => 'en');
			$xsd->text($doc, $self->description());
		}
		my $restriction = $xsd->xs_restriction($xsd->xs_simpleType($element),
			'base' => ($xsd->namespace_prefix() . ':' .
				(defined($self->validate_catalogue())
				? ($xsd->POM()->catalogue->exists($self->validate_catalogue())->to_array())[0]->xs_type()
				: $self->xs_type()))
	   	);
		if (defined($self->validate_catalogue())) 
		{
			my $n = $xsd->POM()->catalogue->exists($self->validate_catalogue())->target_mem_name();
			foreach ($xsd->POM()->catalogue->exists($self->validate_catalogue())->to_array()) 
			{
				my $enum = $xsd->xs_enumeration($restriction, 'value' => $_->$n());
				if ($xsd->configuration()->xmlschema_annotation() && $_->can('description') && defined($_->description()))
				{
					my $doc = $xsd->xs_documentation($xsd->xs_annotation($enum), 'xml:lang' => 'en');
					$xsd->text($doc, $_->description());
				}
			}
		}
		elsif (defined($self->validate()) && $self->validate() =~ /^\^\(.+\|.+\)\$/)
		{
			my @enum = split(/[\(\)\|]/, $self->validate());
			shift(@enum);
			pop(@enum);
			foreach (@enum) {
				$xsd->xs_enumeration($restriction, 'value' => $_);
			}
		}
		elsif (defined($self->validate()))
		{
			$xsd->xs_pattern($restriction, 'value' => $self->validate());
		}
	}
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::Configuration::Abstract;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use Class::STL::ClassMembers;
	use Class::STL::ClassMembers::Constructor;
}

# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::PequelHomePage;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use ETL::Pequel3;
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'pequel_home_page'),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 'http://sourceforge.net/projects/pequel'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Pequel Home Page');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::Version;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use ETL::Pequel3;
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'version'),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => $ETL::Pequel3::VERSION),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'decimal'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Display Pequel Version information');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::Prefix;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'prefix'),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => ''),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Directory pathname prefix');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::XmlParser;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'xml_parser'),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 'default'),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0), #maybe not??
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'NMTOKEN'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Default XML parser type name');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::XmlSchema;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'xml_schema'),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 'default'),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'NMTOKEN'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Default XML schema generator type name');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::XmlSchemaAnnotation;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'xmlschema_annotation'),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'validate', default => '^(0|1)$'),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'byte'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Print annotations in pequel.xsd xml-schema');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::XmlSchemaNSPrefix;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'xmlschema_ns_prefix'),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 'xs'),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'validate', default => '\w+'),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'NMTOKEN'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Namespace prefix for token names in pequel.xsd xml-schema');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::ExecDir;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'exec_dir'),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => './_Pequel'),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Temporary directory for generated programs');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::TraceDir;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'trace_dir'),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => './_Pequel'),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Temporary directory for trace files and generated programs');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::Transfer;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'transfer'),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'value', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'validate', default => '^(0|first|last)$'),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'NMTOKEN'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Copy input fields (including derived fields) to output');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::Hash;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'hash'),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'validate', default => '^(0|1)$'),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'byte'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Generate in memory -- input data can be unsorted');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::ScriptVersion;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'script_version'),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'decimal'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'User pequel script version number.');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::ScriptAuthor;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'script_author'),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => getlogin),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'NMTOKEN'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'User pequel script author name.');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::TabSize;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'tab_size'),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 4),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'validate', default => '^\d+$'),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'integer'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Tab size in characters for generated program');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::TabType;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'tab_type'),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 'spaces'),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'validate', default => '^(hard|spaces)$'),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'NMTOKEN'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Generated program tab type -- hard | spaces');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::DividerChar;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'divider_char'),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => '+'),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'byte'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Generated program divider line character');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::DividerLength;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'divider_length'),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 100),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'integer'),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Generated program divider line length');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::DeclMinPadSize;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'decl_min_pad_size'),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 12),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'integer'),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Generated program minimum pad size for declaration segment');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::NoNulls;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'no_nulls'),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'validate', default => '^(0|1)$'),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'byte'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Do not print zero for null numeric/decimal');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{		
	package ETL::Pequel3::Type::Properties::RmCtrlm;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers;
	use Class::STL::ClassMembers::Constructor;
	sub new_extra
	{
		my $self = shift;
		$self->name('rmctrlm');
		$self->value(0);
		$self->global(0);
		$self->user_program_info(1);
		$self->description('Remove extra ctrl-m from end-of-record on input data stream');
		return $self;
	}
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::On;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'on'),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 'input'),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'NMTOKEN'),
		Class::STL::ClassMembers::DataMember->new(name => 'validate', default => '^(input|output)$');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::PequelType;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'pequel_type'),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 'string'),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'validate_catalogue', default => 'pequel_types'),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'NMTOKEN'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => '');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::DateType;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'date_type'),
#?		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 'config:default_datetype'),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'validate_catalogue', default => 'date_types'),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'token'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Date type');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::TableType;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'table_type'),
#<		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 'external_dynamic'),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'validate_catalogue', default => 'table_types'),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'token'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Table type');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::AggregateType;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'aggregate_type'),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'validate_catalogue', default => 'aggregates'),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'NMTOKEN'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => '');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::DefaultDateType;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'default_datetype'),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 'YYYYMMDD'),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'NMTOKEN'),
		Class::STL::ClassMembers::DataMember->new(name => 'validate_catalogue', default => 'date_types'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Default date format');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::FieldName;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'field_name'),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'token'),
		Class::STL::ClassMembers::DataMember->new(name => 'validate', default => '[_|\w]+');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::SortDirection;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'sort_direction'),
		Class::STL::ClassMembers::DataMember->new(name => 'validate', default => '^(asc|desc)$'),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'NMTOKEN'),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 'asc');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::ShowSynonyms;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'show_synonyms'),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'validate', default => '^(0|1)$'),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'byte'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Print field synonyms in generated code');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::Statistics;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'statistics'),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'validate', default => '^(0|1)$'),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'byte'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Display process benchmark statistics');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::DataSetSpec;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'dataset_spec'),
#<		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 'ascii:STDIN'),
		Class::STL::ClassMembers::DataMember->new(name => 'CDATA', default => 1), # CDATA because may contain complex SQL stmnts;
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'token'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Dataset specification');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::Optimise;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'optimise'),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Optimise generated code');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::Verbose;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'verbose'),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'integer'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Display progress information');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::InputRecordLimit;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'input_record_limit'),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'integer'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Input record process limit count -- process only initial <limit> records');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::BenchmarkOn;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'benchmark_on'),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'validate', default => '^(0|1)$'),
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'byte'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Display program execution benchmark statistics');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::DefaultListDelimiter;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'default_list_delimiter'),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => ','),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'validate', default => '^(,|-|+|_|~|\|| |\\s+)$'), 
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'token'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Default delimiter for list/array fields');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::DefaultDecimalPlaces;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'default_decimal_places'),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => '2'),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'validate', default => '^\d+$'), 
		Class::STL::ClassMembers::DataMember->new(name => 'xs_type', default => 'integer'),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Default decimal place position for amount fields');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::Silent;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'silent'),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'Supress all progress information');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::SortCmd;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'sort_cmd'),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 'sort'),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'External sort command name');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::Sorter;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'sorter'),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => 'unix'),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'External sort command name');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::SortArgs;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'sort_args'),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => ''),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'External sort command pre-set arguments');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::SortTmpDir;
	use base qw(ETL::Pequel3::Type::Properties::Configuration::Abstract);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'name', default => 'sort_tmp_dir'),
		Class::STL::ClassMembers::DataMember->new(name => 'default', default => ''),
		Class::STL::ClassMembers::DataMember->new(name => 'user_program_info', default => 1),
		Class::STL::ClassMembers::DataMember->new(name => 'global', default => 0),
		Class::STL::ClassMembers::DataMember->new(name => 'description', default => 'External sort command temp directory');
	use Class::STL::ClassMembers::Constructor;
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::ScriptName; # TODO ???
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use Class::STL::ClassMembers;
	use Class::STL::ClassMembers::Constructor;
	sub new_extra
	{
		my $self = shift;
		$self->name('script_name');
		$self->description('name of script currently executing');
		$self->value($0);
		$self->global(0);
		$self->user_program_info(1);
		return $self;
	}
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::InlineCc;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use Class::STL::ClassMembers;
	use Class::STL::ClassMembers::Constructor;
	sub new_extra
	{
		my $self = shift;
		$self->name('inline_cc');
		$self->description('inline: CC');
		$self->global(0);
		$self->user_program_info(0);
		return $self;
	}
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::InlineLibs;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use Class::STL::ClassMembers;
	use Class::STL::ClassMembers::Constructor;
	sub new_extra
	{
		my $self = shift;
		$self->name('inline_libs');
		$self->description('inline: LIBS');
		$self->global(0);
		$self->user_program_info(0);
		return $self;
	}
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::InlineInc;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use Class::STL::ClassMembers;
	use Class::STL::ClassMembers::Constructor;
	sub new_extra
	{
		my $self = shift;
		$self->name('inline_inc');
		$self->description('inline: INC');
		$self->global(0);
		$self->user_program_info(0);
		return $self;
	}
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::InlineCcflags;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use Class::STL::ClassMembers;
	use Class::STL::ClassMembers::Constructor;
	sub new_extra
	{
		my $self = shift;
		$self->name('inline_ccflags');
		$self->description('inline: CCFLAGS');
		$self->global(0);
		$self->user_program_info(0);
		return $self;
	}
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::InlineOptimize;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use Class::STL::ClassMembers;
	use Class::STL::ClassMembers::Constructor;
	sub new_extra
	{
		my $self = shift;
		$self->name('inline_optimize');
		$self->description('inline: OPTIMIZE');
		$self->global(0);
		$self->user_program_info(0);
		return $self;
	}
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::InlineLddlflags;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use Class::STL::ClassMembers;
	use Class::STL::ClassMembers::Constructor;
	sub new_extra
	{
		my $self = shift;
		$self->name('inline_lddlflags');
		$self->description('inline: LDDLFLAGS');
		$self->global(0);
		$self->user_program_info(0);
		return $self;
	}
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::InlineMake;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use Class::STL::ClassMembers;
	use Class::STL::ClassMembers::Constructor;
	sub new_extra
	{
		my $self = shift;
		$self->name('inline_make');
		$self->description('inline: MAKE');
		$self->global(0);
		$self->user_program_info(0);
		return $self;
	}
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::InlineCleanAfterBuild;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use Class::STL::ClassMembers;
	use Class::STL::ClassMembers::Constructor;
	sub new_extra
	{
		my $self = shift;
		$self->name('inline_clean_after_build');
		$self->description('inline: CLEAN_AFTER_BUILD');
		$self->global(0);
		$self->user_program_info(0);
		return $self;
	}
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::InlineCleanBuildArea;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use Class::STL::ClassMembers;
	use Class::STL::ClassMembers::Constructor;
	sub new_extra
	{
		my $self = shift;
		$self->name('inline_clean_build_area');
		$self->description('inline: CLEAN_BUILD_AREA');
		$self->global(0);
		$self->user_program_info(0);
		return $self;
	}
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::InlineBuildNoisy;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use Class::STL::ClassMembers;
	use Class::STL::ClassMembers::Constructor;
	sub new_extra
	{
		my $self = shift;
		$self->name('inline_build_noisy');
		$self->description('inline: BUILD_NOISY');
		$self->global(0);
		$self->user_program_info(0);
		return $self;
	}
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::InlineBuildTimers;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use Class::STL::ClassMembers;
	use Class::STL::ClassMembers::Constructor;
	sub new_extra
	{
		my $self = shift;
		$self->name('inline_build_timers');
		$self->description('inline: BUILD_TIMERS');
		$self->global(0);
		$self->user_program_info(0);
		return $self;
	}
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::InlineForceBuild;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use Class::STL::ClassMembers;
	use Class::STL::ClassMembers::Constructor;
	sub new_extra
	{
		my $self = shift;
		$self->name('inline_force_build');
		$self->description('inline: FORCE_BUILD');
		$self->global(0);
		$self->user_program_info(0);
		return $self;
	}
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::InlinePrintInfo;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use Class::STL::ClassMembers;
	use Class::STL::ClassMembers::Constructor;
	sub new_extra
	{
		my $self = shift;
		$self->name('inline_print_info');
		$self->description('inline: PRINT_INFO');
		$self->global(0);
		$self->user_program_info(0);
		return $self;
	}
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::InlineDirectory;
	use base qw(ETL::Pequel3::Type::Properties::Abstract);
	use Class::STL::ClassMembers;
	use Class::STL::ClassMembers::Constructor;
	sub new_extra
	{
		my $self = shift;
		$self->name('inline_directory');
		$self->description('inline: DIRECTORY');
		$self->global(0);
		$self->user_program_info(0);
		return $self;
	}
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::Catalogue::Abstract;
	use base qw(ETL::Pequel3::Type::Catalogue);
	use Class::STL::ClassMembers
		Class::STL::ClassMembers::DataMember->new(name => 'catalogue_name', default => 'properties'),
		Class::STL::ClassMembers::DataMember->new(name => 'target_mem_name', default => 'name'),
		Class::STL::ClassMembers::DataMember->new(name => 'element_type', default => 'ETL::Pequel3::Type::Properties::Abstract');
	use Class::STL::ClassMembers::Constructor;
	use stl;
	use vars qw( $AUTOLOAD );
	sub AUTOLOAD
	{
		my $self = shift;
		(my $prop = $AUTOLOAD) =~ s/.*:://;
		my $p;
		$self->err()->user_error(10601, "Unknown property name '$prop'!")
			unless ($p = $self->exists($prop));
		return $p->value(@_);
	}
	sub DESTROY { }
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties;	#--> ::Catalogue
	use base qw(ETL::Pequel3::Type::Properties::Catalogue::Abstract);
	use Class::STL::ClassMembers;
	use Class::STL::ClassMembers::SingletonConstructor;
	sub new_extra
	{
		my $self = shift;
		$self->push_back( 
			ETL::Pequel3::Type::Properties::PequelHomePage->new(),
			ETL::Pequel3::Type::Properties::Version->new(),
        	ETL::Pequel3::Type::Properties::Hash->new(),
        	ETL::Pequel3::Type::Properties::TabSize->new(),
        	ETL::Pequel3::Type::Properties::TabType->new(),
        	ETL::Pequel3::Type::Properties::DividerChar->new(),
        	ETL::Pequel3::Type::Properties::DividerLength->new(),
        	ETL::Pequel3::Type::Properties::ShowSynonyms->new(),
			ETL::Pequel3::Type::Properties::DeclMinPadSize->new(),
        	ETL::Pequel3::Type::Properties::Statistics->new(),
        	ETL::Pequel3::Type::Properties::Prefix->new(),
        	ETL::Pequel3::Type::Properties::XmlParser->new(),
        	ETL::Pequel3::Type::Properties::XmlSchema->new(),
			ETL::Pequel3::Type::Properties::XmlSchemaAnnotation->new(),
			ETL::Pequel3::Type::Properties::XmlSchemaNSPrefix->new(),
			ETL::Pequel3::Type::Properties::Transfer->new(),
			ETL::Pequel3::Type::Properties::Verbose->new(),
			ETL::Pequel3::Type::Properties::Silent->new(),
			ETL::Pequel3::Type::Properties::Optimise->new(),
        	ETL::Pequel3::Type::Properties::BenchmarkOn->new(),
			ETL::Pequel3::Type::Properties::InputRecordLimit->new(), #TODO -->dataset attribute
        	ETL::Pequel3::Type::Properties::DefaultDateType->new(),
        	ETL::Pequel3::Type::Properties::DefaultListDelimiter->new(),
        	ETL::Pequel3::Type::Properties::DefaultDecimalPlaces->new(),
        	ETL::Pequel3::Type::Properties::DataSetSpec->new(),
        	ETL::Pequel3::Type::Properties::PequelType->new(),
			ETL::Pequel3::Type::Properties::TableType->new(),
			ETL::Pequel3::Type::Properties::DateType->new(),
        	ETL::Pequel3::Type::Properties::AggregateType->new(),
        	ETL::Pequel3::Type::Properties::On->new(),
        	ETL::Pequel3::Type::Properties::FieldName->new(),
        	ETL::Pequel3::Type::Properties::SortDirection->new(),
        	ETL::Pequel3::Type::Properties::SortCmd->new(),
        	ETL::Pequel3::Type::Properties::SortArgs->new(),
        	ETL::Pequel3::Type::Properties::SortTmpDir->new(),
        	ETL::Pequel3::Type::Properties::Sorter->new(),
        	ETL::Pequel3::Type::Properties::ScriptName->new(),
        	ETL::Pequel3::Type::Properties::ScriptVersion->new(),
        	ETL::Pequel3::Type::Properties::ScriptAuthor->new(),
        	ETL::Pequel3::Type::Properties::NoNulls->new(),
        	ETL::Pequel3::Type::Properties::RmCtrlm->new(), # remove -- dataset attribute
        	ETL::Pequel3::Type::Properties::ExecDir->new(),
        	ETL::Pequel3::Type::Properties::TraceDir->new(),
			ETL::Pequel3::Type::Properties::InlineCc->new(), #TODO: put these in inline extension catalogue
			ETL::Pequel3::Type::Properties::InlineLibs->new(),
			ETL::Pequel3::Type::Properties::InlineInc->new(),
			ETL::Pequel3::Type::Properties::InlineCcflags->new(),
			ETL::Pequel3::Type::Properties::InlineOptimize->new(),
			ETL::Pequel3::Type::Properties::InlineLddlflags->new(),
			ETL::Pequel3::Type::Properties::InlineMake->new(),
			ETL::Pequel3::Type::Properties::InlineCleanAfterBuild->new(),
			ETL::Pequel3::Type::Properties::InlineCleanBuildArea->new(),
			ETL::Pequel3::Type::Properties::InlineBuildNoisy->new(),
			ETL::Pequel3::Type::Properties::InlineBuildTimers->new(),
			ETL::Pequel3::Type::Properties::InlineForceBuild->new(),
			ETL::Pequel3::Type::Properties::InlinePrintInfo->new(),
			ETL::Pequel3::Type::Properties::InlineDirectory->new(),
		);
		return $self;
	}
}
# ----------------------------------------------------------------------------------------------------
{
	package ETL::Pequel3::Type::Properties::User;
	use base qw(ETL::Pequel3::Type::Properties::Catalogue::Abstract);
	use Class::STL::ClassMembers;
	use Class::STL::ClassMembers::Constructor;
	sub new_extra
	{
		my $self = shift;
		my $catalogue = ETL::Pequel3::Type::Properties->new();
		foreach (grep($_->global() == 0, $catalogue->to_array())) {
			$self->push_back($_->new());
#>TODO:		$self->push_back($_->global() ? $_ : $_->new()); # so all properties available from configuration;
		}
		return $self;
	}
}
# ----------------------------------------------------------------------------------------------------
1;
